from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck

IMPERSONATION_ROLES = [
    "roles/owner",
    "roles/editor",
    "roles/iam.securityAdmin",
    "roles/iam.serviceAccountAdmin",
    "roles/iam.serviceAccountKeyAdmin",
    "roles/iam.serviceAccountUser",
    "roles/iam.serviceAccountTokenCreator",
    "roles/iam.workloadIdentityUser",
    "roles/dataproc.editor",
    "roles/dataproc.admin",
    "roles/dataflow.developer",
    "roles/resourcemanager.folderAdmin",
    "roles/resourcemanager.folderIamAdmin",
    "roles/resourcemanager.projectIamAdmin",
    "roles/resourcemanager.organizationAdmin",
    "roles/serverless.serviceAgent",
    "roles/dataproc.serviceAgent",
]


class AbsGoogleImpersonationRoles(BaseResourceCheck):
    def __init__(self, name, id, categories, supported_resources):
        super().__init__(name, id, categories, supported_resources)

    def scan_resource_conf(self, conf):
        if 'role' in conf and conf['role'][0] in IMPERSONATION_ROLES:
            return CheckResult.FAILED
        return CheckResult.PASSED



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ELBAccessLogs(BaseResourceCheck):
    def __init__(self):
        name = "Ensure the ELB has access logging enabled"
        id = "CKV_AWS_92"
        supported_resources = ['aws_elb']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        self.evaluated_keys = 'access_logs'
        if 'access_logs' in conf:
            if 'enabled' not in conf['access_logs'][0]:
                return CheckResult.PASSED
            self.evaluated_keys = 'access_logs/[0]/enabled'
            if conf['access_logs'][0]['enabled'] == [True]:
                return CheckResult.PASSED
        return CheckResult.FAILED

check = ELBAccessLogs()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SagemakerEndpointConfigurationEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all data stored in the Sagemaker Endpoint is securely encrypted at rest"
        id = "CKV_AWS_98"
        supported_resources = ['aws_sagemaker_endpoint_configuration']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'kms_key_arn' in conf.keys():
                return CheckResult.PASSED
        return CheckResult.FAILED


check = SagemakerEndpointConfigurationEncryption()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
import json


class S3ProtectAgainstPolicyLockout(BaseResourceCheck):

    def __init__(self):
        name = "Ensure S3 bucket policy does not lockout all but root user. (Prevent lockouts needing root account fixes)"
        id = "CKV_AWS_93"
        supported_resources = ['aws_s3_bucket', 'aws_s3_bucket_policy']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'policy' in conf.keys():
            if isinstance(conf['policy'][0], str):
                try:
                    policy_block = json.loads(conf['policy'][0])
                    if 'Statement' in policy_block.keys():
                        for statement in force_list(policy_block['Statement']):
                            if 'Condition' in statement.keys():
                                # https://github.com/bridgecrewio/checkov/pull/627#issuecomment-714681751
                                continue

                            if 'NotAction' in statement.keys():
                                continue

                            if statement.get('Effect') != 'Deny':
                                continue
                                
                            principal = statement['Principal']

                            if principal == '*':
                                return CheckResult.FAILED
                            elif 'AWS' in statement['Principal']:
                                # Can be a string or an array of strings
                                aws = statement['Principal']['AWS']
                                if (type(aws) == str and aws == '*') or (type(aws) == list and '*' in aws):
                                    return CheckResult.FAILED

                            action = statement['Action']

                            if action == '*':
                                return CheckResult.FAILED
                            elif 's3' in statement['Action']:
                                # Can be a string or an array of strings
                                s3 = statement['Action']['s3']
                                if (type(s3) == str and s3 == '*') or (type(s3) == list and '*' in s3):
                                    return CheckResult.FAILED
                except: # nosec
                    pass
        return CheckResult.PASSED

check = S3ProtectAgainstPolicyLockout()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
import json


class IAMRoleAllowsPublicAssume(BaseResourceCheck):

    def __init__(self):
        name = "Ensure IAM role allows only specific services or principals to assume it"
        id = "CKV_AWS_60"
        supported_resources = ['aws_iam_role']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get('assume_role_policy') and isinstance(conf['assume_role_policy'][0], str):
            try:
                assume_role_block = json.loads(conf['assume_role_policy'][0])
                if 'Statement' in assume_role_block.keys():
                    for statement in assume_role_block['Statement']:
                        if 'Effect' in statement and statement['Effect'] == 'Deny':
                            continue
                        if 'AWS' in statement['Principal']:
                            # Can be a string or an array of strings
                            aws = statement['Principal']['AWS']
                            if (type(aws) == str and aws == '*') or (type(aws) == list and '*' in aws):
                                return CheckResult.FAILED
            except: # nosec
                pass
        return CheckResult.PASSED


check = IAMRoleAllowsPublicAssume()
